//
// C++ Implementation: balkendiagram
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "balkendiagram_h.h"
#include <QPainter>
#include <QInputDialog>
#include <QColorDialog>
#include <QtDebug>

BalkenDiagram_h::BalkenDiagram_h(QWidget* parent): Diagram(parent)
{
	colors[0]=Qt::blue;
	colors[1]=Qt::red;
}


BalkenDiagram_h::~BalkenDiagram_h()
{
}


void BalkenDiagram_h::paintEvent( QPaintEvent *ev)
{
	using namespace std;

	if(data.size()==0) return; // Keine Daten, also nicht zeichnen
	
	QPainter p(this);

	p.fillRect(rect(),Qt::white);
	
	// 1. Maximum finden
	double max = *(max_element(data.begin(),data.end())); // Kurzfassung 
	
	// Zeilen
	int rows,cols;
	if(row_width>0)
	{
		cols = data.size()/row_width;
		rows = row_width;
	}
	else
	{
		cols=1;
		rows = data.size();
	}
	
	int dwidth = width();
	int dminw = 10;
	int dheight = height();
	
	// Titel
	QFont fh(p.font()),fo(p.font());
	fh.setBold(true);
	fh.setPointSize(fh.pointSize()+1);
	QFontMetrics fm(fh);
	p.setFont(fh);
	if(fm.width(_title)>(width()-10)) 
		p.drawText( 5,height()-10,_title);
	else
		p.drawText( (width()-fm.width(_title))/2,height()-10,_title);
	p.setFont(fo);
	dheight -= fm.height()+15;
	
	// Legende
	if (colLabels.count()>0)
	{
		int ms = 0;
		for (int i=0;i<colLabels.count();++i) ms = qMax(ms,p.fontMetrics().width(colLabels[i])); // Maximale Breite bestimmen
		ms += 10; // Rand
		dwidth -= ms+10;
		int h = p.fontMetrics().height()*(colLabels.count())+5;
		QRect rr(dwidth,25,ms,h);
		p.fillRect(rr,Qt::white);
		p.drawRect(rr);
		for (int i=0;i<colLabels.count();++i)
		{
			p.setPen(colors[i%colors.count()]);
			p.drawText(rr.x()+6,rr.y()+(1+i)*p.fontMetrics().height(),colLabels[i]);
		}
	}
	
	// Labelbreite
	QFontMetrics lfm(p.font());	
	int maxll=0;
	for(int i=0;i<labels.count();++i) maxll = qMax(maxll,lfm.width(labels[i]));
	dminw += maxll;
	
	//2. hmax berechnen
	int hmax=dwidth-dminw-80;
	
	//4 b berechnen
	int b;
	if(cols>1) b = dheight/(1.5*(1+cols)*rows);
	else b = dheight/(1.5*rows);
	
	//6. x als Startwert definieren
	
	int y=b*0.643;
	
	int i=0,r=0; // Laufende Nummer
	
	QColor c = colors[0];
	
	p.setPen(Qt::black);
	
	
	for (int x = dwidth-20; x>dminw;x-=(dwidth-40)/5) p.drawLine(x,5,x,dheight);
	
	for(QVector<double>::iterator it = data.begin();it!=data.end();++it)
	{
		double v = *it;
		//3. Vx berechnen
		int vx=v/max*hmax;
		//5. paint rectangle
		p.setPen(Qt::NoPen);
		p.fillRect( dminw,y,vx,b,c);
		QPoint pt[] = {QPoint(dminw,y),QPoint(dminw+vx,y),QPoint(dminw+vx+b*0.343,y-b*0.342),QPoint(dminw+b*0.343,y-b*0.342)};
		p.setBrush(c.light());
		p.drawPolygon(pt,4);
		QPoint ps[] = {QPoint(dminw+vx,y),QPoint(dminw+vx+b*0.343,y-b*0.342),QPoint(dminw+vx+b*0.343,y+b*(1-0.342)),QPoint(dminw+vx,y+b)};
		p.setBrush(c.dark());
		p.drawPolygon(ps,4);
		p.setPen(Qt::black);
		p.setBrush(Qt::white);
		// Beschriften
		const QString t = QString("%1%2").arg(v,0,'f',1).arg(value_postfix);// QString::number( v )+'  '+value_postfix;
		p.drawText( dwidth-20-p.fontMetrics().width(t),y+b/4+10,t); // Wert
		
		if(r==0){
			QString l;
			if(i<labels.count()) l = labels[i]; else l = QString::number(i);
			p.drawText( 1,y+b/4+10,l); // Spalte
		}
		++i;
		
		// 6. naechster x Wert
		if(cols>1) y+=1.5*(1+cols)*b;
		else y+= b*1.5;
		
		// Neue Zeile
		if(row_width>0 && i>=row_width)
		{
			i=0;
			++r;
			y=b*0.843+r*b*1.5;
			c = colors[r%colors.count()];
		}
	}
	p.drawLine( dminw,5,dminw,dheight);
}

void BalkenDiagram_h::colorAction()
{
	int idx=0;
	if(!colLabels.isEmpty())
	{
		bool ok;
		QString l = QInputDialog::getItem(this,tr("Zeile"),tr("Zeile auswhlen deren Farbe sie ndern wollen"),colLabels,0,&ok);
		if(!ok) return;
		idx = colLabels.indexOf(l);
	}
	QColor oc = colors.at(idx%colors.count());
	QColor c = QColorDialog::getColor(oc,this);
	if(c.isValid())	colors[idx%colors.count()]=c;
	update();
}

