//
// C++ Implementation: columnselectdialog
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "columnselectdialog.h"
#include <QStandardItemModel>
#include <QListView>
#include <QLayout>
#include <QDialogButtonBox>
 
ColumnSelectDialog::ColumnSelectDialog(QWidget* parent, Qt::WindowFlags f): QDialog(parent, f),spmodel(0),model(0),lv(0)
{
	spmodel = new QStandardItemModel(this);
	QGridLayout *l = new QGridLayout(this);
	lv = new QListView(this);
	lv->setSelectionMode(QAbstractItemView::ExtendedSelection);
	//lv->setSelectionMode(QAbstractItemView::SingleSelection);
	QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok |QDialogButtonBox::Cancel);
	lv->setModel(spmodel);
	l->addWidget(lv);
	l->addWidget(buttons); 
	connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
	connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
}

ColumnSelectDialog::~ColumnSelectDialog()
{
}

QVector<int> ColumnSelectDialog::getColumnList(QAbstractItemModel *model)
{
	QList<int> r;
	ColumnSelectDialog dlg;
	dlg.setModel(model);
	if(dlg.exec()==QDialog::Accepted)
	{
		r = dlg.columns();
	}
	return r.toVector();
}

int ColumnSelectDialog::getColumn(QAbstractItemModel *model)
{
	ColumnSelectDialog dlg;
	dlg.setModel(model);
	if(dlg.exec()==QDialog::Accepted)
	{
		return dlg.columns().at(0);
	}
	return -1;
}

void ColumnSelectDialog::updateList()
{
	if(!model || !spmodel ) return;
	spmodel->clear();
	for(int i=0;i<model->columnCount();++i)
	{
		QStandardItem *it = new QStandardItem(model->headerData(i,Qt::Horizontal).toString());
		it->setCheckable(false);
		it->setEditable(false);
		spmodel->setItem(i,it);
	}
}

void ColumnSelectDialog::setModel(QAbstractItemModel *m)
{
    model = m;
    updateList();
}


/*!
    \fn ColumnSelectDialog::setMarks(const QVector<int>& marks)
 */
void ColumnSelectDialog::setMarks(const QVector<int>& marks)
{
	for(int i=0;i<spmodel->rowCount();++i)
	{
		QStandardItem *it=spmodel->item(i,0);
		it->setCheckable(true);
		it->setCheckState(Qt::Unchecked);
	}
	for(int i=0;i<marks.count();++i)
	{
		spmodel->item(i,0)->setCheckState(Qt::Checked);
	}
}

/*!
    \fn ColumnSelectDialog::columns()
 */
QList<int> ColumnSelectDialog::columns()
{
	QList<int> lo;
	QList<QModelIndex> l =  lv->selectionModel()->selectedIndexes();
	foreach(QModelIndex i,l)
	{
		lo << i.row();
	}
	return lo;
}
