//
// C++ Implementation: haeufigkeitsdiagrammadapter
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "haeufigkeitsdiagramadapter.h"
#include "diagram.h"
#include <QStandardItemModel>
#include <QAction>
#include <QInputDialog>
#include <QtDebug>

#include <map>

#include "columnselectdialog.h"

HaeufigkeitsDiagramAdapter::HaeufigkeitsDiagramAdapter(QWidget *parent)
 : ModelDiagramAdapter(parent),input_filter(""),filter_spalte(-1)
{
	QAction *a = new QAction(tr("mit Spalte verknpfen"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(depAction()));
	actions << a;
	a = new QAction(tr("Verknpfung aufheben"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(undepAction()));
	actions << a;
}


HaeufigkeitsDiagramAdapter::~HaeufigkeitsDiagramAdapter()
{
}


void HaeufigkeitsDiagramAdapter::updateDiagram()
{
	if(!td || !_model) return;
	using namespace std;

	map<QString,int> he;
	for(int i=0;i<_model->rowCount();++i)
	{
		QString v = _model->data(_model->index(i,model_column)).toString();
		if( filter_spalte>=0 && filter_wert!=_model->data(_model->index(i,filter_spalte)).toString()) continue;
		if( !input_filter.isNull() && input_filter == _model->data(_model->index(i,model_column))) continue;
		he[v]++; // Haeufigkeit zaehlen
	}
	
	QVector<double> sdata(he.size());
	QStringList labels;
	int i=0;
	for(map<QString,int>::iterator it=he.begin();it!=he.end();++it,++i)
	{
		labels << (*it).first;
		sdata[i] = (int)(*it).second;
	}
	
	processData(sdata); // Weiterverarbeiten in abgeleiteter Klasse
	
	td->setLabels(labels);
	td->setData(sdata);
}

void HaeufigkeitsDiagramAdapter::processData(QVector<double>& d)
{
}

void HaeufigkeitsDiagramAdapter::setDepFilter(QString Filter_Wert,int Filter_Spalte)
{
	filter_wert = Filter_Wert;
	filter_spalte = Filter_Spalte;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsDiagramAdapter::setOutputFilter(const QString& fv)
 */
void HaeufigkeitsDiagramAdapter::setOutputFilter(const QVariant& fv)
{
	output_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsDiagramAdapter::setInputFilter(const QString& fv)
 */
void HaeufigkeitsDiagramAdapter::setInputFilter(const QVariant& fv)
{
	input_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsDiagramAdapter::depAction()
 */
void HaeufigkeitsDiagramAdapter::depAction()
{
	bool ok;
	int col=-1;
	if(model()) col = ColumnSelectDialog::getColumn(model());
	if(col<0) return;
	QSet<QString> values;
	for(int i=0;i<model()->rowCount();++i) values << model()->data(model()->index(i,col)).toString();
	
	QString v = QInputDialog::getItem(this,tr("Filter"),tr("Wert nach dem gefiltert werden soll"),values.toList(),0,true,&ok);
	
	if(!ok) return;
	setDepFilter(v,col);
	updateDiagram();
}


/*!
    \fn HaeufigkeitsDiagramAdapter::undepAction()
 */
void HaeufigkeitsDiagramAdapter::undepAction()
{
	setDepFilter(QString(),-1);
}

void HaeufigkeitsDiagramAdapter::saveData(QDataStream& s,int ver)
{
	if(ver>1) ModelDiagramAdapter::saveData(s,ver);
	s << input_filter;
	s << filter_wert << (qint32) filter_spalte;
}

void HaeufigkeitsDiagramAdapter::loadData(QDataStream& s,int ver)
{
	if(ver>1) ModelDiagramAdapter::loadData(s,ver);
	s >> input_filter;
	QString v;
	int sp;
	s >> v >> sp;
	setDepFilter(v,sp);
}
